@@ -4,7 +4,6 @@
 
 from collections import defaultdict
 from math import log
-import time
 import string, itertools
 import multiprocessing
 
@@ -13,19 +12,20 @@ def chunks(l, n):
     """ Yield n successive chunks from l.
     """
     newn = int(1.0 * len(l) / n + 0.5)
-    for i in xrange(0, n - 1):
+    for i in range(0, n - 1):
         yield l[i * newn:i * newn + newn]
     yield l[n * newn - newn:]
 
 
-def wordify_examples(
-        (name_to_table, connecting_tables, context, index_by_value, target_table_name, word_att_length, ex_idxs)):
+def wordify_examples(name_to_table, connecting_tables, context, index_by_value, target_table_name, word_att_length,
+                     ex_idxs):
     cached_sentences = defaultdict(dict)
-
-    return [
-        wordify_example(name_to_table, connecting_tables, context, cached_sentences, index_by_value, target_table_name,
-                        word_att_length, target_table_name, ex, set([])) for ex in
-        name_to_table[target_table_name].getitemsref(ex_idxs)]
+    wordified_examples = []
+    for ex in name_to_table[target_table_name][ex_idxs]:
+        wordified_ex = wordify_example(name_to_table, connecting_tables, context, cached_sentences, index_by_value,
+                                       target_table_name, word_att_length, target_table_name, ex, set([]))
+        wordified_examples.append(wordified_ex)
+    return wordified_examples
 
 
 def wordify_example(name_to_table, connecting_tables, context, cached_sentences, index_by_value, target_table_name,
@@ -40,11 +40,11 @@ def wordify_example(name_to_table, connecting_tables, context, cached_sentences,
     data_name = str(data_name)
 
     if debug:
-        print "======================================"
-        print "example:", ex
-        print "table name:", data_name
-        print "searched_connections:", len(searched_connections), searched_connections
-        print "connecting_tables:", len(connecting_tables[data_name]), connecting_tables[data_name]
+        print("======================================")
+        print("example:", ex)
+        print("table name:", data_name)
+        print("searched_connections:", len(searched_connections), searched_connections)
+        print("connecting_tables:", len(connecting_tables[data_name]), connecting_tables[data_name])
 
     ex_pkey_value = data_name in context.pkeys and ex[str(context.pkeys[data_name])]
 
@@ -52,7 +52,7 @@ def wordify_example(name_to_table, connecting_tables, context, cached_sentences,
 
         words = []  # word list for every example
         if debug:
-            print "words:", len(words)
+            print("words:", len(words))
         # Construct words (tableName_attributeName_attributeValue) from the given table
         for att in name_to_table[data_name].domain.attributes:
             if not str(att.name) in context.pkeys[data_name] and not str(att.name) in context.fkeys[data_name]:
@@ -68,16 +68,16 @@ def wordify_example(name_to_table, connecting_tables, context, cached_sentences,
         for sec_t_name, sec_fkey, prim_fkey in connecting_tables[data_name]:
             sec_t = name_to_table[sec_t_name]
             if debug:
-                print "------------------"
-                print "(sec_t,sec_fkey,prim):", (sec_t_name, sec_fkey, prim_fkey)
-                print "search this table:", not (sec_t_name,
-                                                 sec_fkey) in searched_connections and sec_t_name != target_table_name
-                print "search this table:", not prim_fkey or not (data_name,
-                                                                  sec_fkey) in searched_connections  # and sec_t!=self.target_table
+                print("------------------")
+                print("(sec_t,sec_fkey,prim):", (sec_t_name, sec_fkey, prim_fkey))
+                print("search this table:", not (sec_t_name,
+                                                 sec_fkey) in searched_connections and sec_t_name != target_table_name)
+                print("search this table:", not prim_fkey or not (data_name,
+                                                                  sec_fkey) in searched_connections)  # and sec_t!=self.target_table
             if not (sec_t_name, sec_fkey) in searched_connections and sec_t_name != target_table_name and (
-                not prim_fkey or not (data_name, sec_fkey) in searched_connections):
+                        not prim_fkey or not (data_name, sec_fkey) in searched_connections):
                 example_indexes = index_by_value[sec_t_name][str(sec_fkey)][str(ex_pkey_value)] if not prim_fkey else \
-                index_by_value[sec_t_name][str(prim_fkey)][str(ex[str(sec_fkey)])]
+                    index_by_value[sec_t_name][str(prim_fkey)][str(ex[str(sec_fkey)])]
 
                 for sec_ex_idx in example_indexes:
                     words += wordify_example(name_to_table, connecting_tables, context, cached_sentences,
@@ -160,12 +160,12 @@ class Wordification(object):
 
         p = multiprocessing.Pool(num_of_processes)
 
-        indices = chunks(range(len(self.target_table)), num_of_processes)  # )
+        indices = chunks(list(range(len(self.target_table))), num_of_processes)  # )
 
         for ex_idxs in indices:
-            self.resulting_documents.extend(wordify_examples((self.name_to_table, self.connecting_tables, self.context,
-                                                              self.index_by_value, self.target_table.name,
-                                                              self.word_att_length, ex_idxs)))
+            self.resulting_documents.extend(wordify_examples(self.name_to_table, self.connecting_tables, self.context,
+                                                             self.index_by_value, self.target_table.name,
+                                                             self.word_att_length, ex_idxs))
         p.close()
         p.join()
 
@@ -228,23 +228,26 @@ class Wordification(object):
         for document in self.resulting_documents:
             for word in document:
                 words.add(word)
+        words = sorted(words)
 
         for i, word in enumerate(words):
-            arff_string += "@ATTRIBUTE\t'" + word.replace("'", "") + "'\tREAL\n"
+            arff_string += "@ATTRIBUTE '" + word.replace("'", "") + "' REAL\n"
 
-        arff_string += "@ATTRIBUTE\tclass\t{" + string.join(set([str(a) for a in self.resulting_classes]),
-                                                            ",") + "}\n\n@DATA\n"
+        arff_string += "@ATTRIBUTE class {" + ','.join(set([str(a) for a in self.resulting_classes])) + "}\n\n@DATA\n"
 
+        self.word_features = []
         for doc_idx in range(len(self.resulting_documents)):
             features = []
             for word in words:
+                if word not in self.word_features:
+                    self.word_features.append(word)
                 if word in self.tf_idfs[doc_idx]:
                     features.append(str(self.tf_idfs[doc_idx][word]))
                 else:
                     features.append("0")
             features.append(str(self.resulting_classes[doc_idx]))
 
-            arff_string += string.join(features, ',')
+            arff_string += ','.join(features)
             arff_string += "\n"
 
         return arff_string
@@ -275,8 +278,8 @@ class Wordification(object):
         """
         string_documents = []
         for klass, document in zip(self.resulting_classes, self.resulting_documents):
-            string_documents.append("!" + str(klass) + " " + string.join(document, " "))
-        return string.join(string_documents, "\n")
+            string_documents.append("!" + str(klass) + " " + '' .join(document))
+        return '\n'.join(string_documents)
 
     def att_to_s(self, att):
         """
